package com.ekingwin.bas.cloud.exam.web;

import com.ekingwin.bas.cloud.exam.dto.QuestionVO;
import com.ekingwin.bas.cloud.exam.service.QuestionsService;
import com.ekingwin.bas.cloud.infra.utils.common.JsonUtil;
import com.ekingwin.bas.cloud.test.BaseWebTestService;
import com.ekingwin.bas.cloud.test.util.HttpTestUtils;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;

import java.util.ArrayList;
import java.util.List;

import static org.mockito.ArgumentMatchers.any;

/**
 * @author chenjb
 * @version 1.0.0
 * @ClassName QuestionControllerTest.java
 * @Description TODO
 * @createTime 2021/6/10 18:42
 */
@WebMvcTest(controllers = QuestionController.class)
class QuestionControllerTest extends BaseWebTestService {

    @Test
    void addQuestions() throws Exception {
        HttpTestUtils http = super.build();

        QuestionVO question = new QuestionVO();
        question.setQuestions(new ArrayList<>());
        String content = JsonUtil.objectToJsonString(question);

        Mockito.doReturn(1).when(questionsService).addQuestions(any(List.class));

        ResultActions resultActions = http.post("/addQuestions", content, null, null);

        //可以为虚拟结果添加预期值（这里支持jsonpath，可以根据json路径获取值来进行判断）
        resultActions.andExpect(MockMvcResultMatchers.jsonPath("$.code").value(200));

    }

    @Test
    void queryQuestionsBySubjectId() throws Exception {
        HttpTestUtils http = super.build();

        ResultActions resultActions = http.get("/queryQuestions/1/2/3", null, null, null);

        //可以为虚拟结果添加预期值（这里支持jsonpath，可以根据json路径获取值来进行判断）
        resultActions.andExpect(MockMvcResultMatchers.jsonPath("$.code").value(200));

    }

    @Test
    void editQuestions() throws Exception {
        HttpTestUtils http = super.build();

        QuestionVO question = new QuestionVO();
        String content = JsonUtil.objectToJsonString(question);

        Mockito.doReturn(1).when(questionsService).addQuestions(any(List.class));

        ResultActions resultActions = http.post("/editQuestions", content, null, null);

        //可以为虚拟结果添加预期值（这里支持jsonpath，可以根据json路径获取值来进行判断）
        resultActions.andExpect(MockMvcResultMatchers.jsonPath("$.code").value(200));
    }

    @Override
    protected String getBaseUrl() {
        return "/exam/question";
    }

    @SpringBootApplication
    static class InnerConfig {
    }

    @MockBean
    private QuestionsService questionsService;
}