package com.ekingwin.bas.cloud.exam.web;

import com.ekingwin.bas.cloud.exam.dto.SubjectVerifyVO;
import com.ekingwin.bas.cloud.exam.dto.entity.Subject;
import com.ekingwin.bas.cloud.exam.dto.entity.TestPapers;
import com.ekingwin.bas.cloud.exam.service.ScoreService;
import com.ekingwin.bas.cloud.exam.service.SubjectService;
import com.ekingwin.bas.cloud.exam.service.TestPaperService;
import com.ekingwin.bas.cloud.infra.utils.common.JsonUtil;
import com.ekingwin.bas.cloud.test.BaseWebTestService;
import com.ekingwin.bas.cloud.test.util.HttpTestUtils;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;

import java.util.ArrayList;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;

/**
 * @author chenjb
 * @version 1.0.0
 * @ClassName TestPaperControllerTest.java
 * @Description TODO
 * @createTime 2021/6/10 18:21
 */
@WebMvcTest(controllers = TestPaperController.class)
class TestPaperControllerTest extends BaseWebTestService {

    @Test
    void createTest() throws Exception {
        HttpTestUtils http = super.build();
        TestPapers test = new TestPapers();
        test.setQuestions(new ArrayList<>());
        String content = JsonUtil.objectToJsonString(test);

        Mockito.doReturn(1).when(testPaperService).createTest(any(TestPapers.class));

        ResultActions resultActions = http.post("/createTest", content, null, null);

        //可以为虚拟结果添加预期值（这里支持jsonpath，可以根据json路径获取值来进行判断）
        resultActions.andExpect(MockMvcResultMatchers.jsonPath("$.code").value(200));

        //也可以获取最终响应值，在来进行断言
        MvcResult mvcResult = resultActions.andReturn();
    }

    @Test
    void queryScoreInfo() throws Exception {
        HttpTestUtils http = super.build();
        ResultActions resultActions = http.post("/queryScoreInfo", null, null, null);

        //可以为虚拟结果添加预期值（这里支持jsonpath，可以根据json路径获取值来进行判断）
        resultActions.andExpect(MockMvcResultMatchers.jsonPath("$.code").value(200));

        //也可以获取最终响应值，在来进行断言
        MvcResult mvcResult = resultActions.andReturn();
    }

    @Test
    void deleteTestPapers() throws Exception {
        HttpTestUtils http = super.build();

        Mockito.doReturn(1).when(testPaperService).deleteTestPapers(anyString());

        ResultActions resultActions = http.get("/deleteTestPapers/1,2,3", null, null, null);

        //可以为虚拟结果添加预期值（这里支持jsonpath，可以根据json路径获取值来进行判断）
        resultActions.andExpect(MockMvcResultMatchers.jsonPath("$.code").value(200));

        //也可以获取最终响应值，在来进行断言
        MvcResult mvcResult = resultActions.andReturn();
    }

    @Test
    void queryTestPapers() throws Exception {
        HttpTestUtils http = super.build();

        ResultActions resultActions = http.post("/queryTestPapers", null, null, null);

        //可以为虚拟结果添加预期值（这里支持jsonpath，可以根据json路径获取值来进行判断）
        resultActions.andExpect(MockMvcResultMatchers.jsonPath("$.code").value(200));

        //也可以获取最终响应值，在来进行断言
//        MvcResult mvcResult = resultActions.andReturn();
    }

    @Test
    void getTestPapersDetailsInfo() throws Exception {
        HttpTestUtils http = super.build();

        Mockito.doReturn(new TestPapers()).when(testPaperService).getTestPapersDetailsInfo(anyString());
        ResultActions resultActions = http.get("/getTestPapersDetailsInfo/1", null, null, null);

        //可以为虚拟结果添加预期值（这里支持jsonpath，可以根据json路径获取值来进行判断）
        resultActions.andExpect(MockMvcResultMatchers.jsonPath("$.code").value(200));

    }

    @Test
    void editTest() throws Exception {
        HttpTestUtils http = super.build();
        TestPapers test = new TestPapers();
        test.setQuestions(new ArrayList<>());
        String content = JsonUtil.objectToJsonString(test);

        Mockito.doReturn(1).when(testPaperService).editTest(any(TestPapers.class));

        ResultActions resultActions = http.post("/editTest", content, null, null);

        //可以为虚拟结果添加预期值（这里支持jsonpath，可以根据json路径获取值来进行判断）
        resultActions.andExpect(MockMvcResultMatchers.jsonPath("$.code").value(200));
    }

    @Override
    protected String getBaseUrl() {
        return "/exam/test";
    }

    @SpringBootApplication
    static class InnerConfig {
    }

    @MockBean
    private TestPaperService testPaperService;
    @MockBean
    private ScoreService scoreService;
}